#pragma once

#include <memory>
#include <vector>

class Scene;
class AbstractPainter;
class DrawingSurface;
class TriMesh;

enum class DrawingMode
{
    RASTER,
    GRAPH
};

class DrawingContext
{
public:
    DrawingContext(DrawingSurface* surface, DrawingMode requestedDrawingMode);

    static std::unique_ptr<DrawingContext> Create(DrawingSurface* surface, DrawingMode requestedDrawingMode);

    Scene* getScene() const;

    DrawingSurface* getSurface() const;

    void paint();

private:
    void updateMesh();

    DrawingMode mDrawingMode;
    std::unique_ptr<TriMesh> mMesh;
    std::unique_ptr<Scene> mScene;
    DrawingSurface* mSurface{nullptr};
    std::unique_ptr<AbstractPainter> mPainter;
};

using DrawingContextPtr = std::unique_ptr<DrawingContext>;
