#include "Window.h"

#include "PaintEvent.h"
#include "MouseEvent.h"
#include "KeyboardEvent.h"

#include "Widget.h"
#include "VisualLayer.h"
#include "Scene.h"
#include "TriMesh.h"
#include "AbstractPainter.h"
#include "DrawingContext.h"

#include "IPlatformWindow.h"
#include "Screen.h"
#include "Image.h"

namespace mt
{

Window::Window()
    :   DrawingSurface(),
        mWidget(Widget::Create())
{
    mWidget->setBounds(mWidth, mHeight);
    mWidth = 800;
    mHeight = 600;
}

Window::~Window()
{

}

std::unique_ptr<Window> Window::Create()
{
    return std::make_unique<Window>();
}

void Window::clearPlatformWindow()
{
    mPlatformWindow.reset();
}

void Window::onMouseEvent(const MouseEvent* event)
{
    mWidget->onMouseEvent(event);
}

void Window::onKeyboardEvent(const KeyboardEvent* event)
{
    mWidget->onKeyboardEvent(event);
}

void Window::onPaint(const PaintEvent* event)
{
    mWidget->onPaintEvent(event);
}

void Window::setWidget(WidgetPtr widget)
{
    if (mWidget)
    {
        mWidget.reset();
    }
    mWidget = std::move(widget);
    mWidget->setBounds(mWidth, mHeight);
}

IPlatformWindow* Window::getPlatformWindow() const
{
    if (mPlatformWindow)
    {
        return mPlatformWindow.get();
    }
    return nullptr;
}

void Window::setPlatformWindow(IPlatformWindowPtr window, DrawingMode drawingMode)
{
    mPlatformWindow = std::move(window);
    mDrawingContext = std::make_unique<DrawingContext>(this, drawingMode);
}

void Window::map()
{
    if (mPlatformWindow)
    {
        mPlatformWindow->map();
    }
}

void Window::show()
{
    if (mPlatformWindow)
    {
        mPlatformWindow->show();
    }
}

void Window::doPaint(mt::Screen* screen)
{
    if (mPlatformWindow)
    {
        mPlatformWindow->beforePaint(screen);

        mDrawingContext->getScene()->setLayers(mWidget->getLayers());

        mDrawingContext->paint();

        mPlatformWindow->afterPaint(screen);
    }
}

void Window::clear()
{
    if (mPlatformWindow)
    {
        mPlatformWindow->clear();
    }
}

}
