#pragma once

#include "DrawingSurface.h"

#include <memory>
#include <vector>
#include <cstdint>

class PaintEvent;
class MouseEvent;
class KeyboardEvent;

class DrawingContext;
enum class DrawingMode;
class Widget;
using WidgetPtr = std::unique_ptr<Widget>;
class IPlatformWindow;
using IPlatformWindowPtr = std::unique_ptr<IPlatformWindow>;

namespace mt
{

class Screen;

class Window : public DrawingSurface
{

public:

    Window();

    ~Window();

    static std::unique_ptr<Window> Create();

    void setWidget(WidgetPtr widget);

    void onPaint(const PaintEvent* event);

    void onMouseEvent(const MouseEvent* event);

    void onKeyboardEvent(const KeyboardEvent* event);

    IPlatformWindow* getPlatformWindow() const;

    void setPlatformWindow(IPlatformWindowPtr window, DrawingMode drawingMode);

    void map();

    void show();

    void doPaint(mt::Screen* screen);

    void clear();

    void clearPlatformWindow();

private:
    WidgetPtr mWidget {nullptr};
    IPlatformWindowPtr mPlatformWindow {nullptr};
    std::unique_ptr<DrawingContext> mDrawingContext;
};
}

using WindowUPtr = std::unique_ptr<mt::Window>;
