#include "Button.h"

#include "TextNode.h"
#include "GeometryNode.h"
#include "VisualLayer.h"
#include "MouseEvent.h"

#include <iostream>

Button::Button()
    : Widget(),
      mLabel(),
      mCachedColor(255, 255, 255),
      mClickedColor(Color(180, 180, 180)),
      mClickFunc()
{

}

std::unique_ptr<Button> Button::Create()
{
    return std::make_unique<Button>();
}

void Button::setOnClickFunction(clickFunc func)
{
    mClickFunc = func;
}

void Button::setLabel(const std::string& text)
{
    if (text != mLabel)
    {
        mLabel = text;
        mDirty = true;
    }
}

void Button::onMyMouseEvent(const MouseEvent* event)
{
    if(event->GetAction() == MouseEvent::Action::Pressed)
    {
        mCachedColor = mBackgroundColor;
        setBackgroundColor(mClickedColor);
        if(mClickFunc)
        {
            mClickFunc(this);
        }
    }
    else if(event->GetAction() == MouseEvent::Action::Released)
    {
        setBackgroundColor(mCachedColor);
    }
}

void Button::onPaintEvent(const PaintEvent* event)
{
    if (!needsUpdate())
    {
        return;
    }
    mLayers.clear();

    if (mDirty)
    {
        mMyLayers.clear();
        if(!mVisible)
        {
            return;
        }
        addBackground(event);

        if(!mLabel.empty())
        {
            unsigned fontOffset = unsigned(mLabel.size()) * 4;
            auto middle = DiscretePoint(mLocation.GetX() + mSize.mWidth/2 - fontOffset, mLocation.GetY() + mSize.mHeight/2 + 4);
            auto textLayer = VisualLayer::Create();
            auto textElement = TextNode::Create(mLabel, middle);
            textElement->setFillColor(mBackgroundColor);
            textLayer->setText(std::move(textElement));
            mMyLayers.push_back(std::move(textLayer));
        }
        mDirty = false;
        addMyLayers();
    }

}
