#include "Label.h"

#include "TextNode.h"
#include "GeometryNode.h"
#include "VisualLayer.h"

Label::Label()
    : Widget(),
      mLabel()
{

}

std::unique_ptr<Label> Label::Create()
{
    return std::make_unique<Label>();
}

void Label::setLabel(const std::string& text)
{
    if (text != mLabel)
    {
        mLabel = text;
        mDirty = true;
    }
}

void Label::onPaintEvent(const PaintEvent* event)
{
    if (!needsUpdate())
    {
        return;
    }
    mLayers.clear();

    if (mDirty)
    {
        mMyLayers.clear();
        if(!mVisible)
        {
            return;
        }

        if(!mLabel.empty())
        {
            unsigned fontOffset = unsigned(mLabel.size()) * 4;
            auto middle = DiscretePoint(mLocation.GetX() + mSize.mWidth/2 - fontOffset, mLocation.GetY() + mSize.mHeight/2 + 4);
            auto textLayer = VisualLayer::Create();
            auto textElement = TextNode::Create(mLabel, middle);
            textElement->setFillColor(mBackgroundColor);
            textLayer->setText(std::move(textElement));
            mMyLayers.push_back(std::move(textLayer));
        }
        addMyLayers();
    }
}
